/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|


  Puissance-mètre RF basé sur le AD8310 et le AD8318
  Compilé pour le STM32F103C8
  Électro-Bidouilleur, Novembre 2017
 */

// Constantes
const int voie1_entree = PA0;
const int voie2_entree = PA1;
 
// Variables
char cntr_moyenne;
float voie1_lecture;
float voie2_lecture;
float voie1_puissance;
float voie2_puissance;

// Fonction d'initialisation
void setup() 
{
  // Déclaration des mode de broches du ADC:
  pinMode(voie1_entree, INPUT_ANALOG);
  pinMode(voie2_entree, INPUT_ANALOG);

  // Initialisation du port série
  Serial.begin(9600); // 
}

// Boucle Perpetuelle
void loop()
{ 
  // Faire une moyenne des lectures de ADC pour eliminer le bruit
  for (cntr_moyenne = 1; cntr_moyenne <= 100; cntr_moyenne++)
  { 
    voie1_lecture = ((voie1_lecture * (cntr_moyenne - 1)) + analogRead(voie1_entree)) / cntr_moyenne;
    voie2_lecture = ((voie2_lecture * (cntr_moyenne - 1)) + analogRead(voie2_entree)) / cntr_moyenne;
  }
  // Calculer la puissance en dBm
    // Obtenu de la datasheet du AD8310:
    //    P_in = (V_out / Vslope) + P_offset
    //       et V_slope = 0.0237 V/dB
    //       et P_offset = -95 dB
    // Obtenu de la datasheet du AD8318:
    //    P_in = (V_out / Vslope) + P_offset
    //       et V_slope = -0.0245 V/dB
    //       et P_offset = +22 dB
    // Aussi, compte tenu du ADC de 12 bits au maximum de 3.3 V:
    //     V_out = (ADC / 4095) * V_ref
    //       et V_ref = 3.3 V
  
  voie1_puissance = ((voie1_lecture / 4095 * 3.3) / 0.0230) - 95;
  voie2_puissance = ((voie2_lecture / 4095 * 3.3) / -0.0245) + 22;
 
  //Afficher les resultats au terminal
  Serial.print(voie1_puissance);
  Serial.print("    ");
  Serial.println(voie2_puissance);
  
}
